#ifndef _DIRECTORY_H
#define _DIRECTORY_H


//
// author: Owen Astrachan
// date:  9/21/93
//
// modified 11/28/94
// modified  4/5/95
// modified 1/18/96
//
// classes for manipulating directories
// provide a standard interface for directory
// queries from C++ programs that can, in theory, be implemented
// on several platforms
//
// currently supported: Unix, DOS, Mac in development
//
// the class DirEntry provides directory information
// accessible via methods Name, Size, and IsDir
//
// the class DirStream does I/O on directories
// it supports "standard" (for the Tapestry book)
// iterator methods/member functions
//

// *********  DirEntry member functions:
//
// apstring Name() -- returns name of file
// int Size()    -- returns size of file (in bytes)
// bool IsDir()  -- returns false if NOT directory, else true
//
// DirEntry()    -- constructor, directory entry undefined attributes

// *********  DirStream member functions:
//
// DirStream(apstring name) -- constructor (pass name of directory)
// DirStream()            -- default constructor (use current directory)
// void open(apstring name) -- opens directory stream with given name
// bool fail()            -- returns true if directory operations has
//                           failed, else returns false
// void close()           -- close stream
//
// bool GetEntry(DirEntry &)  -- get an entry
//                           (returns true if success, false if not)
//                           each call gets the "next" entry
//
// void First()           -- set DirStream so first entry is accessible
// bool IsDone()          -- returns true if done, false if not
//                           when false, accessing current entry is ok
// void Next()            -- advance to next entry
// DirEntry Current()     -- return current directory entry
//                           call only when IsDone == false
//


#include <dirent.h>
#include "apstring.h"
#include <sys/types.h>


const apstring DIR_SEPARATOR = "/";

class DirStream;     // need forward reference for friendship

class DirEntry
{
  public:
    DirEntry();                      // constructor
    DirEntry(const apstring & name);   // construct from filename
    ~DirEntry();                     // destructor
    
    apstring Name() const;        // return name (not full path) of file
    apstring Path() const;        // return canonicalized path of file
	 long int Size()    const;        // return size (bytes) of file
    bool IsDir()  const;        // return false if file, true if directory
    
    friend DirStream;     // class has access to internals
        
  private:
    
    // this is the private directory entry information
    // it is platform specific, probably should be a 'handle'
    // to a class PrivateDirEntry defined in directory.cc

    apstring myName;               // NOT full path, just 'file' name
    apstring myPath;               // full, canonicalized path
	 long int mySize;                  // in bytes
    bool myIsDirectory;          // true if directory, else false
};

class DirStream
{

  public:
    DirStream(const apstring & name);  // name is path to directory
    DirStream();                     // current directory
    ~DirStream();                    // destructor
    void open(const apstring & name);  // open, bind to file with name
    void close();                    // close the stream
    bool fail();                     // return true if failed, else false
    bool GetEntry(DirEntry &);       // return false if get fails, else true

    void First();                    // standard iterator functions
    void Next();
    bool IsDone();
    DirEntry Current();
        
    // stuff below is 'esoteric' C++
    //
    // the () method returning void * is what allows
    // the expression: while (dirstream)
    // to work [see Teale, The I/O Stream Handbook]
    
    operator void *() const
    {
        return myStatus ? (void *) this : (void *) 0;
    };

    // allow the expression if (!dirstream)
    
    int operator !() const
    {
        return !myStatus; 
    }

  private:
    
    DIR * myStream;     // this field supports C/UNIX system calls
    int myStatus;       // if 1, then next get will be valid
    apstring myPath;      // full path to this directory
    DirEntry myEntry;   // cached, current entry
    bool myIsClosed;    // 
	    
	 void Initialize(const apstring & s);  // private init commonality
    int Read(DirEntry & dir);           // reading common code

   // disable assignment and copy
 
    DirStream operator = (const DirStream & dir);
    DirStream(const DirStream & dir);
};

#endif /*  _DIRECTORY_H not defined */
