#ifndef _HEART_H
#define _HEART_H

// class for implementing cardioverter/fibrillator simulation
// from an idea by Rich Pattis (in SIGCSE 90)
//
// written by Owen Astrachan
// 9/8/93, modified 1/17/94, modified 6/20/96
//
// use/semantics:
//
// Heart(Monitor & m) -- initialize heart to be monitored
//
// int Sense()    -- precondition: heart still beating
//                   i.e., information still available in file (if reading)
//                   postcondition: returns voltage (int) read from file
//                   exception: returns INT_MAX if no data available 
//
// void Shock(ShockType)  -- causes heart to be shocked, this causes some
//                    action, and may affect subsequent sense readings
//
// bool IsBeating() -- returns true if heart is beating, false otherwise
//
// void SetFile(string name) -- set up heart to read simulated
//                    voltages from file "name"


#include <limits.h>
#include <fstream.h>
#include "apstring.h"
#include "monitor.h"

const int MAX_VOLT = 10;              // maximum voltage for heart beat
const int MIN_VOLT = -10;             // minimum voltage for heart beat

class Heart
{
  public:
    enum ShockType {SlowDown,SpeedUp};
    
    Heart(Monitor & m);       // monitor the heart using monitor m
    void SetFile(const apstring & name); // specifies file for voltages
    int Sense();              // return next sensed heart beat value
    void Shock(ShockType s);  // shock the heart
    bool IsBeating();         // return true if heart is beating

  private:

    bool myFromFile;          // true if beats come from file
    ifstream myFile;          // file being read for simulation
    int mySenseCount;         // store # times heart has been sensed
    bool myBeating;           // stores "good" state (still beating)
    int mySense;              // stores last value read/next value sensed

    Monitor & myMonitor;      // monitor heart via this field
    int SelfSense();          // heart generates its own beat

    Heart (const Heart &);    // pass-by-value prohibited
    Heart & operator = (const Heart &); // no assignment possible	
};

#endif  // _HEART_H not defined
