#ifndef _LIST_H
#define _LIST_H

// a list class that supports insertion at either the front
// or back.  Other operations include testing for containment
// and iterating over all elements of the list.  Iterator
// permits deletion and insertion before current item
//
// mutating functions, change list
//
// void Append(const Type & t)   -- add t to back of list
// void Prepend(const Type & t)  -- add t to front of list
// void ChopFront()              -- remove first element
// void Clear()                  -- make the list empty
// 
// accessing functions, get values from list
//
// Type Front()     -- return copy of first element in list
// Type Back()      -- return copy of last element in list
// bool Contains(const Type & t) -- return true iff list contains t
// int Size(), int Length() -- return # elements in list
// void Print()             -- print list elements
//
// iterating functions:
// First(), IsDone(), Next() -- iterate through all elements
// Type & Current()          -- return element: can modify element
//                              (const version exists too)
// Delete()                     -- delete current item 
// InsertBefore(const Type & t) -- insert t before current item

// #include "bool.h"

template <class Type>
class List
{
  public:
    
    List();                                // make an empty list
    List(const List & list);               // copy constructor
    ~List();                               // free memory

    const List& operator = (const List & list);  // assignment operator
    
    // mutators, add values to list
    
    void Append(const Type & t);           // new value at end
    void Prepend(const Type & t);          // new value at front
    void ChopFront();                      // chop off first value in list
    void Clear();                          // make the list empty
    
    // accessors, determine properties of list, get first/last values

    Type Front() const;                    // return copy of first element
    Type Back()  const;                    // return copy of last element
                
    bool Contains(const Type & t) const;   // true if t in list
    int Size() const;                      // # of items in list
    int Length() const;                    // # of items in list
    void Print() const;                    // print elements in list
        
    // iterator functions, traverse list, delete/modify
    
    void First() const;                    // set iterator to front
    bool IsDone() const;                   // finished iterating?
    void Next() const;                     // advance to next item
    
    Type& Current();                       // return current item
    const Type& Current() const;           // return const current item

    void Delete();                         // delete current item
    void InsertBefore(const Type & t);     // add t before current item

  private:

    struct TNode
    {
        // data members
        Type info;              // value stored
        TNode * next;           // link to next TNode
        
        // constructors 
        TNode()
            : next(0)                     
        { }
        TNode(const Type & val, TNode * link=0)
            : info(val),
              next(link)
        { }
        
        
    };
    
    TNode * myFirst;          // first node of list
    TNode * myLast;           // last node of list
    int myCount;              // # of items in list
    
    TNode * myCurrent;        // used by iterator
    TNode * myBefore;         // one before current
    bool myCurrentValid;      // true if current valid
};

#endif
