#include <iostream.h>
#include <fstream.h>
#include "apmatrix.h"
#include "apvector.h"
#include "apstring.h"
#include "iomanip.h"

// program to read inter-city distances into a matrix
// of distances
// illustrates two-dimensional arrays, file processing
// Owen Astrachan, 7/11/95

// the RoadMap class stores city names and the distances between
// cities:
//
//          city A   city B   city C
// city A     0       15       20
// city B    15        0       17
// city C    20       17        0
//
// currently the Read member function assumes data is in the format
// given in the files cities.dat from the Stanford Graphbase as
// given in Knuth, The Stanford Graph Base, Addison-Wesley, 1993
//
// city-name[latitude,longitude]population-in-1980
// d1 d2 d3 ... d(n-1)
//
// where d's are distances to cities appearing earlier in file.  The
// first three cities from cities.dat are reproduced below
//
//   Youngstown, OH[4110,8065]115436
//   Yankton, SD[4288,9739]12011
//   966
//   Yakima, WA[4660,12051]49826
//   1513 2410
//
// in general, the distances do NOT appear on a single line
// but are separated by whitespace
// 
class RoadMap
{
  public:
    RoadMap();
    void Read(const apstring & filename);     // read from specified file
    void PrintCities();                     // all cities printed
    void PrintDistances();                  // all distances printed
    void FindMaxDistance();                 // max distance between cities
    int NumCities();                        // return number of cities
    int Distance(const apstring & c1,         // return distance between
                 const apstring & c2);        // named cities
  private:
    int CityIndex(const apstring & name);     // find index of given city
    const int MY_MAX_CITIES;                // maximum capacity (# cities)
    apmatrix<int> myDistance;                 // 2-d array of distances
    apvector<apstring> myCities;                // list of city names
    int myCityCount;                        // # of cities stored
};

RoadMap::RoadMap()
  : MY_MAX_CITIES(130),
    myCities(MY_MAX_CITIES),
    myDistance(MY_MAX_CITIES,MY_MAX_CITIES),
    myCityCount(0)
// postcondition: all fields initialized, # cities = 0     
{
    // all work done in initializer list
}

void RoadMap::Read(const apstring & filename)
// precondition: filename is name of readable file
// postcondition: data read from file and stored
{
    int count = 0;
    ifstream input;
    input.open(filename.c_str());
    if (! input)
    {
        cerr << "could not open " << filename << " for reading" << endl;
    }
    else
    {
        apstring dummy;
        int j;
        while(getline(input,myCities[count],'[')) // city name and state
        {
            getline(input,dummy);                 // discard rest of line
            for(j=count-1; j >= 0; j--)           // once for each city
            {             
                input >> myDistance[count][j];    // read/store distance
                myDistance[j][count] = myDistance[count][j]; 
            }
            if (count != 0)
            {
                getline(input,dummy);             // discard last '\n'
            }
            myDistance[count][count] = 0;         // no garbage values
            count++;                              // another city read
        }
        myCityCount = count;
    }
}

void RoadMap::PrintCities()
// postcondition: all cities printed     
{
    int j;
    for(j=0; j < myCityCount; j++)
    {
        cout << myCities[j] << endl;
    }
    cout << endl << "total # cities: " << myCityCount << endl;
}
    
void RoadMap::PrintDistances()
// postcondition: distances between each city printed (triangular form)
{
    int j,k;
        
    for(j=0; j < myCityCount; j++)
    {
        for(k=0; k <= j; k++)
        {
            cout << setw(5) << myDistance[j][k];
        }
        cout << endl;
    }
}
    
void RoadMap::FindMaxDistance()
// postcondition: maximal distance between two
//                cities calculated and printed with names of cities     
{
    int j,k;
    int maxfrom = 0;     // index of 'from' city
    int maxto = 0;       // index of 'to' city
    int maxdistance = 0; // maximal distance between two cities
        
    for(j=0; j < myCityCount; j++)
    {
        for(k=0; k < j; k++)
        {
            if (myDistance[j][k] > maxdistance)   // update from/to/dist
            {
                maxfrom = j;
                maxto = k;
                maxdistance = myDistance[j][k];
            }
        }
    }
    cout << endl << "maximal distance in map: "
         << myDistance[maxfrom][maxto] << endl;
    cout << "from " << myCities[maxfrom] << " to "
         << myCities[maxto] << endl;
}
    
int RoadMap::NumCities()
// postcondition: returns # of cities in RoadMap     
{
    return myCityCount;
}
    
int RoadMap::Distance(const apstring & c1, const apstring & c2)
// precondition: c1 and c2 are names of cities for which data are stored
// postcondition: distance between c1 and c2 returned
//                if c1 or c2 not stored, then -1 returned     
{
    int fromIndex = CityIndex(c1);
    int toIndex = CityIndex(c2);
        
    if (fromIndex != -1 && toIndex != -1)       // both cities found?
    {
        return myDistance[fromIndex][toIndex];
    }
    else
    {
        return -1;
    }
}
    
int RoadMap::CityIndex(const apstring & name)
// precondition: name is stored in RoadMap
// postcondition: returns index in vector myCities of name
//                returns -1 if not stored     
{
    int j;
    for(j=0; j < myCityCount; j++)
    {
        if (myCities[j] == name)
        {
            return j;
        }
    }
    return -1;
}
    
int main()
{
    apstring name;
    apstring city1,city2;
    RoadMap map;
        
    cout << "name of input file: ";
    getline(cin,name);
        
    map.Read(name);
    map.PrintCities();
    map.PrintDistances();
    map.FindMaxDistance();

    while(true)
    {
        cout << "enter name of city (stop to exit) ";
        getline(cin,city1);

        // *** loop exit ***
        if (city1 == "stop") break;

        cout << "enter another city name ";
        getline(cin,city2);

        cout << "distance between " << city1 << " and ";
        cout << city2 << " = " << map.Distance(city1,city2) << endl;
    }    
    return 0;
}
